/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DiscRackGateway.java,v 1.11 2004/02/01 04:17:45 christianc Exp $
 */
package org.enhydra.barracuda.discRack;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import com.lutris.appserver.server.Application;
import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.StandardApplication;
import com.lutris.logging.StandardLogger;
import com.lutris.util.Config;
import com.lutris.util.ConfigFile;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.apache.log4j.*;

import org.enhydra.barracuda.discRack.pres.screens.*;
//import org.enhydra.barracuda.examples.ex2.ConfigScreen;

import org.enhydra.instantdb.jdbc.idbDriver;

/**
 * <p>Application Gateway for the Barracuda DiscRack example. Couple of
 * special notes regarding deployment.
 *
 * <p>1. This example was built using Enhydra 3.1 and XMLC 1.2. For
 *    best results, use these versions or higher.
 *
 * <p>2. This example uses Instant DB and the standard DODS generated
 *    data classes from previous DiscRack examples. To make it work,
 *    you will need to add 3 jar files to the enhydra classpath:
 *      <ul>
 *       <li>idb.jar (v3.21)</li>
 *       <li>jdbc2_0-stdext.jar</li>
 *       <li>jta-spec1_0_1.jar</li>
 *      </ul>
 *
 * <p>We can't just put these files in /Web-inf/lib because DODs is
 *    dependant on the enhydra Application interface, and the
 *    StandardApplication implementation requires these jar files
 *    be part of the enhydra classpath
 *
 * <p>3. You must manually build/populate the IDB databases in /Web-inf/db.
 *    To do this use the idb ScriptTool on sql1.txt (in the db directory)
 *
 * <p>To understand this example, you should really look at the class diagram
 * which illustrates how everythign fits together.
 * <a href="sm_barracudaDiscRack.gif">You can view it here...</a>
 *
 */
public class DiscRackGateway extends ApplicationGateway {

    //public constants
    public static final String ENHYDRA_APPLICATION = DiscRackGateway.class.getName()+"EnhydraApplication";
    protected static Logger logger = Logger.getLogger(DiscRackGateway.class.getName());

    //private vars
    private static final String CONFIG_FILE = "ConfFile";
//  private ServletConfig config = null;
    private LoginScreen lnkLoginScreen;                     //added by TJ to show association
    private RegistrationScreen lnkRegistrationScreen;       //added by TJ to show association
    private DiscScreen lnkDiscScreen;                       //added by TJ to show association



    //-------------------- ApplicationGateway --------------------
    /**
     * Perform any local initialization (this is where you should
     * specify any known EventGateways)
     */
    public void initializeLocal() {
        //set up the enhydra app for DODs stuff
//      this.config = config;
        setUpEnhydraApplication();

        //specify the event gateways
        this.specifyEventGateways(new LoginScreen());
        this.specifyEventGateways(new RegistrationScreen());
        this.specifyEventGateways(new DiscScreen());
    }


    /**
     * Set up an underlying Enhydra application for database
     * related stuff (DODS needs this to function properly).
     * We'll store a reference to the Application in the
     * ServletContext, so if anyone needs access to it, they
     * can get it.
     */
//  private void setUpEnhydraApplication(ServletConfig config) {
    private void setUpEnhydraApplication() {
        //get the context and see if the app is already set
        ServletContext context =  this.getServletConfig().getServletContext();
        Application app = (Application) context.getAttribute(ENHYDRA_APPLICATION);

        //if not, create it...
        if (app==null) {
            try {
                //create the underlying enhydra Application
                if (logger.isDebugEnabled()) logger.debug("Creating underlying Enhydra Application");
                app = new DiscRackApplication();

                //set the log channel
                app.setLogChannel(new StandardLogger(true).getChannel(""));

                //get the config file
                String configFile = this.getServletConfig().getInitParameter(CONFIG_FILE);
                if (logger.isDebugEnabled()) logger.debug("Config:"+configFile);
//              InputStream is = this.getClass().getResourceAsStream(configFile);
                InputStream is = context.getResourceAsStream(configFile);
                if (is==null) is = this.getClass().getResourceAsStream(configFile);
                Config conf = new ConfigFile(is).getConfig();
                if (logger.isDebugEnabled()) logger.debug("Got Config file:"+conf);

                //startup the app
                app.startup(conf);
                if (logger.isDebugEnabled()) logger.debug("Started app:"+app);

                //save it in the context
                context.setAttribute(ENHYDRA_APPLICATION, app);
            } catch(Exception e) {
                System.out.println("Error setting up Enhydra Application:"+e);
                e.printStackTrace();
            }
        }
    }

    /**
     * Handle all HTTP requests
     */
    public void handleDefault (HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        ServletContext context =  this.getServletConfig().getServletContext();
        Application app = (Application) context.getAttribute(ENHYDRA_APPLICATION);
        try {
            if (logger.isDebugEnabled()) logger.debug("\n\n"+req.getRequestURI()+" @"+Integer.toHexString(req.hashCode()));
            Enhydra.register(app);
            super.handleDefault(req, resp);
        } finally {
            Enhydra.unRegister();
        }
    }

    class DiscRackApplication extends StandardApplication {

    }

}
